/*
 * Tira-2.1(*) IR code capture program
 *  (*) http://www.home-electro.com/tira2.php
 *
 * 2005.06 Programmed by T.Adachi (tadachi@tadachi-net.com)
 *
 * Reference: Document of "Tira-2 serial protocol".
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>

#include "tira2_serial.h"

int
main(int argc,
     char *argv[])
{
  FILE *fp;
  int fd,ir_cap_length,ircode_length,write_bytes;
  struct termios savetty;   // for restoring tty settings
  unsigned char ir_cap_buf[IRCODE_BUF_LEN];
  unsigned char ircode[IRCODE_BUF_LEN];

  /** check args **/
  if (argc!=2) {
    fprintf(stderr,"Usage: tira2_rec2 <write filename(IR data)>\n");
    exit(1);
  }

  /** open serial port **/
  if (tira_open_serial(TIRA2_SERIAL_PORT,&fd) != 0) exit(1);

  /** change attribute for Tira2 **/
  if (tira_init_attribute(fd,&savetty) != 0) exit(1);

  /** change to six bytes mode **/
  if (tira_change_6bytes_mode(fd) != 0) exit(1);

  /** chack Tira2 **/
  if (tira_check_tira2(fd) != 0) exit(1);

  /** switch to timing mode **/
  if (tira_switch2timing_mode(fd) != 0) exit(1);

  /** capture IR data **/
  if (tira_capture_IR_code(fd,ir_cap_buf,&ir_cap_length) != 0) exit(1);

  /** generate ir code */
  if (tira_generate_ircode(ir_cap_buf,ir_cap_length,
			   ircode,&ircode_length) !=0) exit(1);

  /** close serial port **/
  if (tira_close_serial(fd,&savetty) != 0) exit(1);

  /** write the IR code **/
  fp = fopen(argv[1],"wb");
  if (fp == NULL) {
    fprintf(stderr,"ERROR: file write error!.\n");
  }
  write_bytes = fwrite(ircode,1,ircode_length,fp);
  fclose(fp);

  exit(0);
}
