/*
 * Tira-2.1(*) IR code capture/transmit program
 *  (*) http://www.home-electro.com/tira2.php
 *
 * 2005.06 Programmed by T.Adachi (tadachi@tadachi-net.com)
 *
 * Reference: Document of "Tira-2 serial protocol".
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>

#include "tira2_serial.h"

int
tira_open_serial(const char *serial_port_name,
		 int *fd)
{
  /** open serial port **/
  printf("Opening serial port: %s ...",serial_port_name);
  //tadachi  *fd = open(serial_port_name, O_RDWR | O_NDELAY | O_NONBLOCK | O_NOCTTY);
  *fd = open(serial_port_name, O_RDWR | O_NOCTTY);
  if (*fd < 0) {
    fprintf(stderr,"ERROR: Can't open %s [%s]\n",
	    serial_port_name,strerror(errno));
    return -1;
  }
  printf("OK.\n");
  return 0;
}

int
tira_init_attribute(const int fd,
		    struct termios *savetty)
{
  struct termios tty;

  /** set tty attribute **/
  printf("Setting serial port attribute...");
  if (tcgetattr(fd, &tty) != 0) {      // read serial attribute
    fprintf(stderr,"ERROR: read attribute error [%s]\n",
	    strerror(errno));
    return -1;
  }
  *savetty = tty; // for restore

  cfmakeraw(&tty);                     // see TERMIOS(3)
  cfsetospeed(&tty, B9600);            // set output baud rate to 9600bps
  //cfsetispeed(&tty, B9600);            // set input baud rate to 9600bps
  tty.c_cflag |= CRTSCTS | CS8 | CLOCAL | CREAD;
  tty.c_cflag |= CRTSCTS;              // RTS-CTS hardware flow control
  tty.c_lflag = 0; // set input mode: non-canonical, no echo...
  tty.c_cc[VTIME] = 0;  // charactor timer = 0
  tty.c_cc[VMIN] = 2;  // device block during 2 charactors are recived.

  tcflush(fd, TCIFLUSH); // flush FIFO
  
  if (tcsetattr(fd, TCSADRAIN, &tty) != 0) { // set tty attribute
    fprintf(stderr,"ERROR: set attribute error [%s]\n",strerror(errno));
    return -1;
  }
  printf("OK.\n");
  return 0;
}

int
tira_change_6bytes_mode(const int fd)
{
  unsigned char response_buf[RESPONSE_BUF_LEN];
  int rc;

  /** checking Tira-2 **/
  printf("Changing 6byte mode...");
  // clear buffered serial data
  tcflush(fd, TCIFLUSH);

  // check Tira (switch "six bytes" mode). dummy
  if (write(fd,"IR",2) != 2) {
    //error
    fprintf(stderr,"ERROR: set 6bytes mode [%s]\n",strerror(errno));
    return -1;
  }
  usleep (2 * (100 * 1000));
  memset (response_buf, 0, RESPONSE_BUF_LEN);
  rc = read(fd,response_buf,RESPONSE_BUF_LEN-1);
  if (strncmp((char*)response_buf,"OK",2) != 0) {
    //error
    fprintf(stderr,"ERROR: set 6bytes mode [%s]\n",strerror(errno));
    return -1;
  }
  printf("OK.\n");
  tcflush(fd, TCIFLUSH);
  return 0;
}

int
tira_check_tira2(const int fd)
{
  unsigned char response_buf[RESPONSE_BUF_LEN];
  int rc;

  // check Tira-2 
  printf("Checking Tira type...");
  if (write(fd,"IP",2) != 2) {
    //error
    fprintf(stderr,"ERROR: check tira [%s]\n",strerror(errno));
    return -1;
  }
  usleep (2 * (100 * 1000));
  read(fd,response_buf,RESPONSE_BUF_LEN-1);
  if (strncmp((char*)response_buf,"OIP",3) != 0) {
    //error
    fprintf(stderr,"ERROR: check tira [%s]\n",strerror(errno));
    return -1;
  }
  if (!(response_buf[4] & 0xf0)) {
    fprintf(stderr,"Ira/Tira-1 detected.\n");
    fprintf(stderr,"You can use this program by Tira-2 only!\n exit.\n");
    return -1;
  }
  printf("Tira-2 detected.\n");
  tcflush(fd, TCIFLUSH);

  // check firmware version of Tira-2
  printf("Checking Tira-2 firmware version... ");
  write (fd, "IV", 2);
  usleep (2 * (100 * 1000));
  memset (response_buf, 0, RESPONSE_BUF_LEN);
  rc = read (fd, response_buf, RESPONSE_BUF_LEN-1);
  printf(" firmware vesion:<%s>\n",response_buf);
  tcflush(fd, TCIFLUSH);
  return 0;
}

int
tira_switch2timing_mode(const int fd)
{
  unsigned char send_buf[SEND_BUF_LEN];
  unsigned char response_buf[RESPONSE_BUF_LEN];

  // switch timing mode...
  printf("Switching timing mode... ");
  memset(send_buf,0,4);
  sprintf((char*)send_buf,"IC"); // 'I' 'C' 0x00 0x00
  if (write(fd,send_buf,4) != 4) {
    fprintf(stderr,"ERROR: change timing mode [%s]\n",strerror(errno));
    return -1;
  }
  usleep (2 * (100 * 1000));
  memset (response_buf, 0, RESPONSE_BUF_LEN);
  read(fd,response_buf,RESPONSE_BUF_LEN);
  if (strncmp((char*)response_buf,"OIC",3) != 0) {
    fprintf(stderr,"ERROR: change timing mode [%s]\n",strerror(errno));
    return -1;
  }
  printf("OK.\n");
  tcflush(fd, TCIFLUSH);
  return 0;
}

int
tira_capture_IR_code(const int fd,
		     unsigned char *ir_data,
		     int *irdata_length)
{
  unsigned char response_buf[RESPONSE_BUF_LEN];
  int read_cnt,i,read_times,captured_data;

  //usleep(1000000);
  tcflush(fd, TCIFLUSH);
  printf("Capturing IR-code...\nPlease push remote controller button.\n");

  memset(ir_data,0,IRCODE_BUF_LEN);
  *irdata_length = 0;

  read_times = 0;
  captured_data = -1;
  while(read_times<10) {
    // clear buffer
    memset(response_buf, 0, RESPONSE_BUF_LEN);
    //    tcflush(fd, TCIFLUSH);
    // wait 500ms (0.5sec)
    usleep(500000);
    read_times++;
    // read serial data
    read_cnt = read(fd,response_buf,RESPONSE_BUF_LEN);

    if (read_cnt > 0) {
      for (i=0;i<read_cnt;i++) {
	ir_data[*irdata_length] = response_buf[i];
	(*irdata_length) ++;
	if (*irdata_length >= RESPONSE_BUF_LEN) {
           fprintf(stderr,"ERROR: capture data overflow! (>=%d)\n",RESPONSE_BUF_LEN);
           return -1;
        }
      }
#ifdef DEBUG
      printf("\n--\n");
      for (i=0;i<read_cnt;i++) {
	printf("0x%02x ",(unsigned char)response_buf[i]);
      }
      printf("\n--\n");
#endif
      // check the mark of end of the capturing
      if ((response_buf[read_cnt-4] == 0x00)&&
	  (response_buf[read_cnt-3] == 0x00)&&
	  (response_buf[read_cnt-1] == 0xb2)) {
	  printf("...OK.\n");
	  captured_data = 0;
	  break;
      }
    } else if (read_cnt == 0) {
      // no-data
      continue;
    } else {
      printf("...%d",read_times);
      fflush(stdout);
    }
  } // end of while loop

  if (captured_data == 0) {
    printf("IR-Code captured!\n");
  } else {
    fprintf(stderr,"\nERR: IR-Code capture failed!\n");
  }
  return captured_data;
}

int
tira_close_serial(const int fd,
		  struct termios *restore_tty)
{
  printf("Closing serialport.\n");
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd, TCSADRAIN, restore_tty);   // restore settings
  close(fd);
  return 0;
}

int
tira_generate_ircode(unsigned char *cap_data,
		     int cap_data_length,
		     unsigned char *ircode,
		     int *ircode_length)
{
  unsigned char cw_data;

  printf("Generating IR-code... ");
  cw_data = cap_data[cap_data_length-2];

  if (tira_encode_ir_code(cap_data,cap_data_length-4,
			  &(ircode[2]),ircode_length) != 0) return -1;

  *ircode_length += 2;
  ircode[0] = cw_data;
  ircode[1] = 0x00;
  printf("OK.\n");

  return 0;
}

int
my_compare_func(const void* val1_ptr,
	     const void* val2_ptr)
{
  short s_val1 = *(unsigned short*)val1_ptr;
  short s_val2 = *(unsigned short*)val2_ptr;

  if (s_val1 < s_val2) return -1;
  if (s_val1 == s_val2) return 0;
  return 1;
}


int
tira_transfer_IR_code(const int fd,
		      unsigned char *ircode,
		      int ircode_length)
{
  unsigned char send_buf[SEND_BUF_LEN];
  unsigned char response_buf[RESPONSE_BUF_LEN];

  printf("Transfer IR code... ");
  tcflush(fd,TCIOFLUSH);
  strcpy((char*)send_buf,"IX");
  memcpy(&(send_buf[2]),ircode,ircode_length);
  if (write(fd,send_buf,ircode_length+2) != ircode_length+2) {
    fprintf(stderr,"ERROR: Transfer IR code (data) [%s]\n",strerror(errno));
    return -1;
  }
  usleep(500000);
  memset (response_buf, 0, RESPONSE_BUF_LEN);
  read(fd,response_buf,RESPONSE_BUF_LEN);
  if (strncmp((char*)response_buf,"OIX",3) != 0) {
    fprintf(stderr,"ERROR: Transfer IR code [%s]\n",strerror(errno));
    return -1;
  }
  printf("OK.\n");
  tcflush(fd, TCIFLUSH);
  return 0;
}

int
tira_encode_ir_code(unsigned char *cap_data,
		    int cap_data_length,
		    unsigned char *tr_ircode,
		    int *ircode_length)
{
  unsigned short *rec_ircode_work,*ircode_uniq_work,*ircode_sort_work;
  int i,j,uniq_cnt,find_flag;
#ifdef DEBUG
  int k;
#endif

  rec_ircode_work = (unsigned short*)malloc(cap_data_length+1);
  ircode_sort_work = (unsigned short*)malloc(cap_data_length);
  ircode_uniq_work = (unsigned short*)malloc(cap_data_length);

  // pack the rec_ircode datas by 2bytes
  for (i=0;i<cap_data_length;i+=2) {
    ircode_sort_work[i/2] = rec_ircode_work[i/2]
      = (cap_data[i]<<8)+cap_data[i+1];
    if (ircode_sort_work[i/2] == 0) {
      fprintf(stderr,"ERROR: Captured data is wrong.\n");
      return -1;
    }
  }

#ifdef DEBUG
      printf("===debug info org <%d>===\n",cap_data_length/2);
      for (k=0;k<cap_data_length/2;k++) {
	printf("%04d:%04X ",k,rec_ircode_work[k]);
      }
      printf("\n\n");
      printf("================\n");
#endif      

  // *** This algorithm was not optimized so well :-)
  // *** If you have any good and simple ideas, please let me know.

  // In order to build BSA from all captured timing data. And the
  // BSA can be defined only to 12 pieces. Therefore, I considered
  // that the difference to 40us was the same signal and pack it
  // for one BSA. The single unit is 8us. So, the value in which
  // the difference of 40us is shown becomes five.
  // The datasheet(Tira-2 serial protocol) said the value that
  // considered to be the same is range of about 17-56us. 
  // If you want more information and more detail, see the datasheet.

  // sort them for searching unique values.
  qsort(ircode_sort_work,cap_data_length/2,sizeof(short),my_compare_func);

  // The same(range) value is filled by zero. 
  for (i=0;i<cap_data_length/2;i++) {
    for (j=i+1;j<cap_data_length/2;j++) {
      if ((ircode_sort_work[j] >= ircode_sort_work[i]-5)&&
	  (ircode_sort_work[j] <= ircode_sort_work[i]+5)) {
	ircode_sort_work[j] = 0;
      }
    }
  }

  // pick up unique value
  qsort(ircode_sort_work,cap_data_length/2,sizeof(short),my_compare_func);
  uniq_cnt = 0;
  for (i=0;i<cap_data_length/2;i++) {
    if (ircode_sort_work[i] != 0) {
      ircode_uniq_work[uniq_cnt] = ircode_sort_work[i];
      uniq_cnt++;
    }
  }

  // if the BSA code is overflow, return with error code.
  if (uniq_cnt>12) {
    fprintf(stderr,"ERROR: BSA overflow\n");
    return -1;
  }

  // replace to BSA ID
  for (i=0;i<cap_data_length/2;i++) {
    find_flag = 0;
    for (j=0;j<uniq_cnt;j++) {
      if ((rec_ircode_work[i] >= ircode_uniq_work[j]-5)&&
          (rec_ircode_work[i] <= ircode_uniq_work[j]+5)) {
	rec_ircode_work[i] = j;
        find_flag = 1;
	break;
      }
    }
    if (!find_flag) {
#ifdef DEBUG
      printf("===debug info <i:%d> <j:%d> <%d> <%04X> <%04X>===\n",i,j,uniq_cnt,rec_ircode_work[i],ircode_uniq_work[j-1]);
      int k;
      for (k=0;k<cap_data_length/2;k++) {
	printf("%04d:%04X ",k,rec_ircode_work[k]);
      }
      printf("\n\n");
      for (k=0;k<uniq_cnt;k++) {
	printf("<%d>: %04X ",k,ircode_uniq_work[k]);
      }
      printf("================\n");
#endif      
      fprintf(stderr,"ERROR: internal error.\n");
      return -1;
    }
  }

  // add terminator
  rec_ircode_work[cap_data_length/2] = 0xf;

  // make BSA data(s)
  for (i=0;i<uniq_cnt;i++) {
    tr_ircode[i*2] = (ircode_uniq_work[i] & 0xff00)>>8;
    tr_ircode[i*2+1] = (ircode_uniq_work[i] & 0x00ff);
  }
  // if the number of BSA is under 12, fill zero.
  for (;i<12;i++) {
    tr_ircode[i*2] = tr_ircode[i*2+1] = 0;
  }

  // pack two IR code to one byte.
  for (i=0;i<cap_data_length/2;i+=2) {
    tr_ircode[(i/2)+12*sizeof(short)] =
      (char)(((0x000f & rec_ircode_work[i])<<4) | (0x000f & rec_ircode_work[i+1]));
  }

  *ircode_length = (cap_data_length)/2/2+1+sizeof(short)*12;

  free(rec_ircode_work);
  free(ircode_sort_work);
  free(ircode_uniq_work);

  return 0;
}
