/*
 * Tira-2.1(*) IR code transfer program
 *  (*) http://www.home-electro.com/tira2.php
 *
 * 2005.06 Programmed by T.Adachi (tadachi@tadachi-net.com)
 *
 * Reference: Document of "Tira-2 serial protocol".
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>

#include "tira2_serial.h"

int
main(int argc,
     char *argv[])
{
  FILE *fp;
  int i,fd,ircode_length;
  struct termios savetty;   // for restoring tty settings
  unsigned char ircode[IRCODE_BUF_LEN];

  /** check args **/
  if (argc<2) {
    fprintf(stderr,"Usage: tira2_trans2 <load filenames(IR data)>\n");
    exit(1);
  }

  /** open serial port **/
  if (tira_open_serial(TIRA2_SERIAL_PORT,&fd) != 0) exit(1);

  /** change attribute for Tira2 **/
  if (tira_init_attribute(fd,&savetty) != 0) exit(1);

  /** change to six bytes mode **/
  if (tira_change_6bytes_mode(fd) != 0) exit(1);

#ifdef CHECK_TIRA2
  /** chack Tira2 **/
  if (tira_check_tira2(fd) != 0) exit(1);
#endif

  for (i=1;i<argc;i++) {
    /** read IR code **/
    fp = fopen(argv[i],"rb");
    if (fp == NULL) {
      fprintf(stderr,"ERROR: file read error!\n");
      exit(1);
    }
    ircode_length = fread(ircode,1,IRCODE_BUF_LEN,fp);
    // it needs CW+0x00+24+code bytes
    if (ircode_length < 27) {
      fprintf(stderr,"ERROR: The file is not IR-code file!\n");
      exit(1);
    }
    fclose(fp);

    /** transfer IR data **/
    if (tira_transfer_IR_code(fd,ircode,ircode_length) != 0) exit(1);

    /* wait */
    if (i != argc-1) usleep(200000);
  }

  /** close serial port **/
  if (tira_close_serial(fd,&savetty) != 0) exit(1);

  exit(0);
}
